@@ -33,9 +33,11 @@ package com.twelvemonkeys.imageio.plugins.svg;
 import com.twelvemonkeys.image.ImageUtil;
 import com.twelvemonkeys.imageio.ImageReaderBase;
 import com.twelvemonkeys.imageio.util.IIOUtil;
+import com.twelvemonkeys.lang.StringUtil;
 import org.apache.batik.anim.dom.SVGDOMImplementation;
 import org.apache.batik.anim.dom.SVGOMDocument;
 import org.apache.batik.bridge.*;
+import org.apache.batik.css.parser.CSSLexicalUnit;
 import org.apache.batik.dom.util.DOMUtilities;
 import org.apache.batik.ext.awt.image.GraphicsUtil;
 import org.apache.batik.gvt.CanvasGraphicsNode;
@@ -47,6 +49,7 @@ import org.apache.batik.transcoder.*;
 import org.apache.batik.transcoder.image.ImageTranscoder;
 import org.apache.batik.util.ParsedURL;
 import org.apache.batik.util.SVGConstants;
+import org.apache.batik.xml.LexicalUnits;
 import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 import org.w3c.dom.svg.SVGSVGElement;
@@ -76,7 +79,12 @@ import java.util.Map;
  * @see <A href="http://www.mail-archive.com/batik-dev@xml.apache.org/msg00992.html">batik-dev</A>
  */
 public class SVGImageReader extends ImageReaderBase {
+
+    final static boolean DEFAULT_ALLOW_EXTERNAL_RESOURCES =
+            "true".equalsIgnoreCase(System.getProperty("com.twelvemonkeys.imageio.plugins.svg.allowexternalresources"));
+
     private Rasterizer rasterizer;
+    private boolean allowExternalResources = DEFAULT_ALLOW_EXTERNAL_RESOURCES;
 
     /**
      * Creates an {@code SVGImageReader}.
@@ -113,6 +121,9 @@ public class SVGImageReader extends ImageReaderBase {
         if (pParam instanceof SVGReadParam) {
             SVGReadParam svgParam = (SVGReadParam) pParam;
 
+            // set the external-resource-resolution preference
+            allowExternalResources = svgParam.isAllowExternalResources();
+
             // Get the base URI
             // This must be done before converting the params to hints
             String baseURI = svgParam.getBaseURI();
@@ -324,24 +335,54 @@ public class SVGImageReader extends ImageReaderBase {
             }
 
             // ----
+            SVGSVGElement rootElement = svgDoc.getRootElement();
 
             // get the 'width' and 'height' attributes of the SVG document
-            Dimension2D docSize = ctx.getDocumentSize();
-            if (docSize != null) {
-                defaultWidth = (float) docSize.getWidth();
-                defaultHeight = (float) docSize.getHeight();
+            UnitProcessor.Context uctx
+                    = UnitProcessor.createContext(ctx, rootElement);
+            String widthStr = rootElement.getAttributeNS(null, SVGConstants.SVG_WIDTH_ATTRIBUTE);
+            String heightStr = rootElement.getAttributeNS(null, SVGConstants.SVG_HEIGHT_ATTRIBUTE);
+            if (!StringUtil.isEmpty(widthStr)) {
+                defaultWidth = UnitProcessor.svgToUserSpace(widthStr, SVGConstants.SVG_WIDTH_ATTRIBUTE, UnitProcessor.HORIZONTAL_LENGTH, uctx);
             }
-            else {
-                defaultWidth = 200;
-                defaultHeight = 200;
+            if(!StringUtil.isEmpty(heightStr)){
+                defaultHeight = UnitProcessor.svgToUserSpace(heightStr, SVGConstants.SVG_HEIGHT_ATTRIBUTE, UnitProcessor.VERTICAL_LENGTH, uctx);
             }
-            SVGSVGElement rootElement = svgDoc.getRootElement();
-            String viewBoxStr = rootElement.getAttributeNS
-                    (null, SVGConstants.SVG_VIEW_BOX_ATTRIBUTE);
-            if (viewBoxStr.length() != 0) {
-                float[] rect = ViewBox.parseViewBoxAttribute(rootElement, viewBoxStr, null);
-                defaultWidth = rect[2];
-                defaultHeight = rect[3];
+
+            boolean hasWidth = defaultWidth > 0.0;
+            boolean hasHeight = defaultHeight > 0.0;
+
+            if (!hasWidth || !hasHeight) {
+                String viewBoxStr = rootElement.getAttributeNS
+                        (null, SVGConstants.SVG_VIEW_BOX_ATTRIBUTE);
+                if (viewBoxStr.length() != 0) {
+                    float[] rect = ViewBox.parseViewBoxAttribute(rootElement, viewBoxStr, null);
+                    // if one dimension is given, calculate other by aspect ratio in viewBox
+                    // or use viewBox if no dimension is given
+                    if (hasWidth) {
+                        defaultHeight = defaultWidth * rect[3] / rect[2];
+                    }
+                    else if (hasHeight) {
+                        defaultWidth = defaultHeight * rect[2] / rect[3];
+                    }
+                    else {
+                        defaultWidth = rect[2];
+                        defaultHeight = rect[3];
+                    }
+                }
+                else {
+                    if (hasHeight) {
+                        defaultWidth = defaultHeight;
+                    }
+                    else if (hasWidth) {
+                        defaultHeight = defaultWidth;
+                    }
+                    else {
+                        // fallback to batik default sizes
+                        defaultWidth = 400;
+                        defaultHeight = 400;
+                    }
+                }
             }
 
             // Hack to work around exception above
@@ -608,6 +649,14 @@ public class SVGImageReader extends ImageReaderBase {
             public void displayMessage(String message) {
                 processWarningOccurred(message.replaceAll("[\\r\\n]+", " "));
             }
+
+            @Override
+            public ExternalResourceSecurity getExternalResourceSecurity(ParsedURL resourceURL, ParsedURL docURL) {
+                if (allowExternalResources) {
+                    return super.getExternalResourceSecurity(resourceURL, docURL);
+                }
+                return new NoLoadExternalResourceSecurity();
+            }
         }
     }
 }
